/* takes the Cryosat ocean waveform and then retracks the leading edge */

/*-- standard C headers --*/
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <string.h>

/*--
   routine to take the Cryosat ocean waveform and re-track the
   leading edge
   
   input
     nw       -   odd number of waveforms in the least squares fit
     sigw     -   sigma of Gaussian weight function used in fit
     rw       -   128 samples of the waveform
     iwf      -   (0) - use waveform  (1) - don't use waveform
     t0       -   initial estimate of arrival time (1-128)
     sig0     -   initial estimate of pulse width
     npar     -   (2) - solve for amp and t1 (3) - also solve for sig1
     ming     -   minimum gate for least squares fit
     maxg     -   maximum gate for least squares fit
     alph     -   trailing edge decay parameter - different for each satellite

   output
     rmw      -   model waveform
     t1       -   updated arrival time (1-128)
     sig1     -   updated pulse width
     amp      -   model amplitude
     std      -   standard deviation of misfit (< 0 -  something went wrong)
     prob     -   F-test probability that variance reduction is due
                  to random fluctuations (< 0.2 keep iterating)

--*/

int erfpick_(nw,sigw,rw,iwf,t0,sig0,npar,ming,maxg,alph,rmw,t1,sig1,amp,std,prob)
int 	*nw; /*---input parameters passed as pointers---*/
double	*sigw;
double	*rw;
int	*iwf;
double	*t0;
double	*sig0;
int	*npar;
double  *alph;
int     *ming;
int     *maxg;
double	*rmw; /*---output parameters to pass as pointers---*/
double	*t1;
double	*sig1;
double	*amp;
double	*std;
double	*prob;
{

/*-- dereference the pointers to variables that are internal to the function --*/

	/*-- input parameters --*/
	int	npari=*npar;
	int	mx=128*(*nw);

/*-- internal variables --*/
	int	dpt, gate, info, nc, ndata, np, nt, parnc, parnr, wavno, nw2;
	double	chi1, chi2, eta, ef, ex2, f, suma, sumb, summ, sumw, talph, sigw2, time, wfac, z;
	double	ex[128], dmds[mx], dmdt[mx], rm[mx];
	double	a[mx][3];
	double	b[mx], w[mx];
	double	ata[3][3];
	double	atb[3];
	char	uplo;
	
/*-- set some constants --*/
	double	damp=0.9;
	double	pi=4.0*atan(1.0);
	double	sq2=sqrt(2.0);
	double	sqp=sqrt(pi);
	
/*-- set mingate and maxgate --*/
	int	mingate; 
	int	maxgate;
        
	nw2 = (*nw-1)/2;
        mingate = *ming;
        maxgate = *maxg;

/*-- make sure there is enough memory --*/
	
	if ((*nw)*128 > mx) {
		fprintf(stderr,"Increase mx in erf_retrack");
	}
	
/*-- don't let the swh get less than 0.54 m --*/
	if(*sig0 < 0.3){
		*sig0=0.3;
	}
		
/*-- construct the initial model and partial derivatives --*/

	for(wavno = 0; wavno<*nw; wavno++){
		for( gate=0; gate<128; gate++){
			dpt = 128*(wavno)+gate;
			time = ((double)gate)+1.0;
			talph = fmax(0.0,(time-t0[wavno]+8.0)*(*alph)); /*-- talph is different --*/
			eta = (time-t0[wavno])/(sq2*(*sig0));
			ef = erf(eta); /*-- value same as in Fortran --*/
			ex[gate] = exp(-talph); /*-- value differs from Fortran --*/
			ex2 = exp(-eta*eta);
			rm[dpt] = ex[gate]*(1.0+ef)/2.0; /*-- test if these three are the same for Fortran and C --*/
			dmdt[dpt] = -ex2/((*sig0)*sq2*sqp);
			dmds[dpt] = -eta*ex2/((*sig0)*sqp);
		}
	}
	
/*-- estimate the amplitude --*/

	sumw = 0.0;
	summ = 0.0;
	nc=((*nw+1)/2)-1;
	sigw2 = (*sigw)*(*sigw);
	
/*-- Initialize A, b, and w --*/
	
/*-- this weight function depends on the distance to the center of the 
	window. also make this large if this waveform is flagged as bad. --*/
	
	for(wavno = 0; wavno<*nw; wavno++){
		wfac=exp(((double)(wavno-nc)*(double)(wavno-nc))/(2.0*sigw2));
		if(iwf[wavno] == 1) {
			wfac=100000.0;
		}
		
		for(gate = mingate; gate<maxgate+1; gate++){
			dpt = 128*(wavno)+gate;

/*-- The division by 6 is to get a correct chi squared. There are 100 waveforms
	averaged for each 20 Hz waveform. However only 36(??) of these are independent.
	Let's guess 72. --*/
	
			w[dpt] = wfac*(rw[dpt]+5500.0)/9.80;
			sumw = sumw + ((rw[dpt]*rm[dpt]))/(w[dpt]*w[dpt]);	
			summ = summ + ((rm[dpt]*rm[dpt]))/(w[dpt]*w[dpt]);	
		}
	}
	
	*amp = sumw/summ;

/*-- Modify and subtract the starting model and construct the A-matrix 
	 (which is the diagonal weight matrix W multiplied to the Jacobian matrix J
	 (which in turn is contains partial derivatives of the model function 
	 with respect to the model parameters)) --*/
	 
	 chi1  = 0.0;
	 	 
	 for(wavno = 0; wavno<*nw; wavno++){
	 	for(gate = mingate; gate<maxgate+1; gate++){
	 		dpt = 128*(wavno)+gate;
	 		b[dpt] = (rw[dpt] - (*amp)*rm[dpt])/w[dpt];
	 		chi1 = chi1 + (b[dpt]*b[dpt]);
	 		a[dpt][0] = rm[dpt]/w[dpt];
	 		a[dpt][1] = (*amp)*dmdt[dpt]/w[dpt];
	 		a[dpt][2] = (*amp)*dmds[dpt]/w[dpt];
	 	}
	 }

/*-- Compute (A^T)A and (A^T)b --*/

	/*-- (A^T)b --*/
	for(parnr = 0; parnr<*npar; parnr++){
	
		sumb = 0.0;
		
		for(wavno = 0; wavno<*nw; wavno++){
	 		for(gate = mingate; gate<maxgate+1; gate++){
				dpt = 128*(wavno)+gate;
				sumb = sumb + (a[dpt][parnr]*b[dpt]);
			}
		}
		
		atb[parnr] = sumb;

/*-- (A^T)A --*/	
	for(parnc = parnr; parnc<*npar; parnc++){

		suma = 0.0;
		
		for(wavno = 0; wavno<*nw; wavno++){
	 		for(gate = mingate; gate<maxgate+1; gate++){
				dpt = 128*(wavno)+gate;
				suma = suma + (a[dpt][parnr]*a[dpt][parnc]);
			}
		}
		ata[parnc][parnr] = suma; /*-- This is the transpose of the actual result --*/
	}
	}
		
/*-- Solve the problem --*/
	
	uplo='U';
	np=3;
	nt=1;
	
	dposv_(&uplo,npar,&nt,ata,&np,atb,&np,&info);
/*-- add comments here about the arguments --*/

/*-- construct the solution --*/
	if(info == 0){
		*amp = *amp+damp*atb[0];
		
		for(wavno = 0; wavno<*nw; wavno++){
			t1[wavno] = t0[wavno]+damp*atb[1];
		}
		
		if(npari == 2){
			*sig1 = *sig0;
		} else {
			*sig1 = *sig0+damp*atb[2];
		}
	
/*-- check for absurd waveforms --*/	
		if(abs((t1[nw2]-32.5)>20.0) | (*amp < 5000.0) | (*amp > 200000.0) | (*sig1 < 0.1)){
			*std = -9.0;
			return;
		}
	
/*-- generate the final model --*/
	for(wavno = 0; wavno<*nw; wavno++){
		for( gate=0; gate<128; gate++){
			dpt = 128*(wavno)+gate;
			time = (double)gate+1.0;
			talph = fmax(0.0,(time-t1[wavno]+8.0)*(*alph)); 
			eta = (time-t1[wavno])/(sq2*(*sig1));
			ef = erf(eta);
			ex[gate] = exp(-talph);
			ex2 = exp(-eta*eta);
			rm[gate] = (ex[gate]*(1.0+ef)/2.0);
			rmw[dpt]=rm[gate]*(*amp);
		}
	}
	
/*-- evaluate the misfit --*/
		chi2 = 0.0;
		ndata = 0;
		for(wavno = 0; wavno<*nw; wavno++){
	 		for(gate = mingate; gate<maxgate+1; gate++){
	 			dpt = 128*(wavno)+gate;
	 			ndata = ndata+1;
	 			chi2 = chi2 + ((rw[dpt]-rmw[dpt])*(rw[dpt]-rmw[dpt])/(w[dpt]*w[dpt]));			
	 		}
	 	}
		*std = sqrt(chi2/(ndata-npari));
		f = chi2/chi1;
		z = (f-1.046511628)/0.323222961;
		*prob = (1.0+erf(z*sq2))/2.0;
	} else {
		*std = -abs(info);
	}
	
	return EXIT_SUCCESS;
}
