/*-- Reads in waveform data records (wdr) in the "c20" format specified by the
	structure in cryosat20hz.h then computes 1-second statistics --*/ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main( int   argc, char *argv[] )
{

	double mean(int, double[]);
	double median(int, double[]);
	double stdev(int, double[]);
	void linreg(int, double[], double[], double[]);

	int		nwav = 20; /* use 22 for CryoSat and 20 for Envisat and Jason */
	int		tide, delay, hgt, h0, issb, ioff, boff, nstr, tide_csr, bline;
	int		loop, wavno, ns=0;
	
	double	lat[nwav], lon[nwav], trec[nwav];
	double	ylin[nwav], hmdiff[nwav], hmdiffdet[nwav], swh20[nwav];
	double	slat, swh, slon, tnow, tprev, hstdev, hmed, stime;
	
	struct CRYOSAT20HZ c;
	FILE	*fpin;

	if (argc != 4 || (fpin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "usage:  c20_1stat  input.c20 nstruct baseline\n");
        fprintf (stderr, "        nstruct (1) - use CryoSat on-board range \n");
        fprintf (stderr, "        nstruct (2) - use 2-parameter retracking result \n");
        fprintf (stderr, "        nstruct (3) - use 3-parameter retracking result \n");
        fprintf (stderr, "        nstruct (4) - use threshold retracking result \n");
        fprintf (stderr, " \n");
        fprintf (stderr, "        baseline (1) - Baseline A \n");
        fprintf (stderr, "        baseline (2) - Baseline B \n");
		return EXIT_FAILURE;
	}
	nstr = atoi(argv[2]);
	bline = atoi(argv[3]);
	
	// Error handling:
	if(nstr != 1 && nstr != 2 && nstr != 3 && nstr != 4){
		fprintf(stderr,"invalid nstruct value \n");
		goto endpoint;
	}
	
	if(bline != 1 && bline != 2){
		fprintf(stderr,"invalid bline value \n");
		goto endpoint;
	}
	
	
	tprev = 0.0;
	
	while( loop == 1 ){
		loop = 0;
		
		for(wavno=0; wavno<nwav; wavno++){
			if((fread ((void *)&c, sizeof(struct CRYOSAT20HZ), (size_t)1, fpin)) != 1){
				/* fprintf(stderr, "Error reading record from file \n"); */
				goto endpoint; 
			}
			tnow = 1.0e-06*(double)c.microsec+(double)c.sec2000;
			
			/* detect breaks in time between passes */
			if ((ns != 0) & ((tnow-tprev) > 4.0)) {
				goto endloop;
			}
					
			/* editing criteria for retracking errors */
			if(c.trackbits != 0 ) {
				goto endloop;
	  		}
	  		
			lat[wavno] = 1.0e-06 * (double)c.lat;
			lon[wavno] = 1.0e-06 * (double)c.lon;
			trec[wavno] = tnow;
		
			tide = c.hltide + c.hstide + c.hptide + c.hotide;
        	tide_csr = c.new_tide + c.hstide;

			delay = c.hwet + c.hdry + c.hiono + c.hinvb;
        	ioff = -3800; /* based on analysis by Remko Scharroo at 2011 Crypsat meeting */
        	boff = 0;
        	issb = floor(0.060*c.pswh[1]+.5);
			h0 =c.alt-c.range; 
			
			hgt = 0.0;
			
			if (abs(c.hinvb) < 1 | abs(c.hdry) < 1 | abs(c.hwet) < 1 ) {
				goto endloop;	
			}
						
			switch ( nstr ) {
				case 1:
					hgt = (double)h0; 
					break;
				case 2:
					hgt = (double)h0 - (double)c.drange[1];
					break;
				case 3:
					hgt = (double)h0 - (double)c.drange[0];
					break;
				case 4:
					hgt = (double)h0 - (double)c.drange[2];
					break;
			}
			
			if (bline == 2) {
				hgt += boff;
			}

        	
        	hmdiff[wavno] = (double)(hgt-c.mss);
            swh20[wavno] = c.pswh[0];
        	tprev=tnow;
		}
		
		slat = median(nwav, lat);
		slon = median(nwav, lon);
		stime = median(nwav, trec);
        swh = median(nwav, swh20);
        
        linreg(nwav, hmdiff, lat, ylin);
        
        for(wavno=0; wavno<nwav; wavno++){ 
			hmdiffdet[wavno] = hmdiff[wavno] - ylin[wavno];
		}
        
		hmed = median(nwav, hmdiff);
		hstdev = stdev(nwav, hmdiffdet);

		/* factor to account for the multiple waveform retracking.
   		on average there are 1.27 points used so we should multiply all numbers by 1.12 */
   		
   		
   		if(nstr == 2 || nstr == 3) {
   			hstdev *= 1.12;
   		}
		
		for(wavno=0; wavno<nwav; wavno++){ 
			hmdiffdet[wavno] = hmdiff[wavno] - ylin[wavno];
		}

		fprintf(stdout,"%lf %lf %lf %lf %lf %lf \n", stime, slon, slat, hstdev, hmed, swh);
		
		endloop:
		tprev=tnow;
		loop = 1;
		ns++;
	}
	
	endpoint:
	fclose (fpin);
	fprintf(stderr, "Number of 1 Hz records: %d \n", ns);
	return EXIT_SUCCESS;
}

double mean(int nw, double htd[]) {
    double	sum=0;
    int		i;
    
    for(i=0; i<nw; i++)
        sum = sum + htd[i];
    return((double)sum/nw);
}
 
double median(int nw, double sld[]) {
    double 	temp;
    int 	i, j;
    // the following two loops sort the array x in ascending order
    for(i=0; i<nw-1; i++) {
        for(j=i+1; j<nw; j++) {
            if(sld[j] < sld[i]) {
                // swap elements
                temp = sld[i];
                sld[i] = sld[j];
                sld[j] = temp;
            }
        }
    }
 
    if(nw%2==0) {
        // if there is an even number of elements, return mean of the two elements in the middle
        return((sld[nw/2] + sld[nw/2 - 1]) / 2.0);
    } else {
        // else return the element in the middle
        return sld[nw/2];
    }
}

double stdev(int nw, double htd[]) {
    double	sum=0.0, mht=0.0;
    int		i;
    
    mht = mean(nw, htd); 
    
    for(i=0; i<nw; i++)
        sum = sum + ((htd[i]-mht)*(htd[i]-mht));
    return(sqrt((double)sum/(nw-1)));

}

void linreg(int nw, double yi[], double xd[], double ye[]) {
	double my=0.0, mx=0.0;
	double slope, sloped=0.0, slopen=0.0, inter;
	int i;

	my = mean(nw,yi);
	mx = mean(nw,xd);
	
	for(i=0; i<nw; i++) {
		slopen = slopen + ((xd[i]-mx)*(yi[i]-my));
		sloped = sloped + ((xd[i]-mx)*(xd[i]-mx));
	}
	
	slope = slopen/sloped;
	inter = my - (slope*mx);
	
	
	
	for(i=0; i<nw; i++) {
		ye[i] = inter + (slope*xd[i]);
	}
}
