/*-- Reads in waveform data records (wdr) in the "c20" format specified by the
	structure in cryosat20hz.h then computes 1-second statistics --*/ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main( int   argc, char *argv[] )
{

	double mean(int, double[]);
	double median(int, double[]);

	int		nwav = 22;
	int		nwi = nwav-1;
	int		tide, delay, h, h0, issb, kk, ioff, nstr, tide_csr;
	int		loop, wavno, swdiff, ns=0;
	
	double	lat[nwav], lon[nwav], t[nwav];
	double	ha[nwav], hmdiff[nwav], hdsq[nwav], sloped[nwi], adslopes[nwi];
	double	slat, slon, st, ssloped, tnow, tprev, mhdiff, mad, stdev;
	
	double	alt, range, ssh, diff, wfac, sfac=0., damp;
	struct CRYOSAT20HZ c;
	FILE	*fpin;

	if (argc != 3 || (fpin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "usage:  c20_1stat  input.c20 nstruct  \n");
        fprintf (stderr, "        nstruct (1)-use CryoSat on-board range \n");
        fprintf (stderr, "        nstruct (2)-use 2-parameter retracking result \n");
        fprintf (stderr, "        nstruct (3)-use 3-parameter retracking result \n");
        fprintf (stderr, "        nstruct (4)-use threshold retracking result \n");
		return EXIT_FAILURE;
	}
	nstr = atoi(argv[2]);
	
	tprev = 0.0;
	while( loop == 1 ){
		loop = 0;
		
		for(wavno=0; wavno<nwav; wavno++){
			if((fread ((void *)&c, sizeof(struct CRYOSAT20HZ), (size_t)1, fpin)) != 1){
				/* fprintf(stderr, "Error reading record from file \n"); */
				goto endpoint; 
			}
			swdiff = abs(c.pswh[1]-c.pswh[0]);
			tnow = 1.0e-06 * c.microsec + c.sec2000;
			
			/* detect breaks in time between passes 
			if (ns != 0 & (tnow-tprev) > 4.0) {
				tnow = tprev;
				goto endloop;
			} */
					
			/* editing criteria for retracking errors */
			if(c.trackbits != 0 ) goto endloop;
	  		
	  		
			lat[wavno] = 1.0e-06 * (double)c.lat;
			lon[wavno] = 1.0e-06 * (double)c.lon;
		
			/* select geographic area */
			/* if(lat[wavno] > 5.0 | lat[wavno] < -55.0 ) goto endloop; */
		
			tide = c.hltide + c.hstide + c.hptide + c.hotide;
        	tide_csr = c.new_tide + c.hstide;

			delay = c.hwet + c.hdry + c.hiono + c.hinvb;
        	ioff = -3800.0; /* based on analysis by Remko Scharroo at 2011 Crypsat meeting */
        	issb = floor(0.060*c.pswh[1]+.5);
			h0 =c.alt-(c.range+c.hdopp+tide_csr+delay+ioff)+issb; 
			
	  		if(nstr == 1) h = h0;
   			if(nstr == 2) h = h0 - c.drange[1];
			if(nstr == 3) h = h0 - c.drange[0];
        	if(nstr == 4) h = h0 + c.range_s;
        	
        	hmdiff[wavno] = (double)(h-c.mss);
        	ha[wavno] = h;
        	
        	tprev=tnow;
		}
		
		/* compute slopes 
		for(wavno=1; wavno<nwav; wavno++){
			sloped[wavno-1]=(hmdiff[wavno]-hmdiff[wavno-1])/(111.19*(lat[wavno]-lat[wavno-1]));
		} */
		
		
		slat = median(nwav, lat);
		slon = median(nwav, lon);
		
		mhdiff = mean(nwav, hmdiff);
		/* ssloped = median(nwi, sloped); */

		
		/* compute absolute deviation at 1 Hz
		fprintf(stderr,"%lf ",ssloped);
		for(wavno=0; wavno<nwi; wavno++){
			adslopes[wavno]=abs(sloped[wavno] - ssloped);
			fprintf(stderr,"%lf ",adslopes[wavno]);
		}
		mad = median(nwi, adslopes);
		fprintf(stderr,"%d %lf \n",wavno, mad); */
		
		/* compute mean square height anomalies at 1 Hz*/
		for(wavno=0; wavno<nwav; wavno++){
			hdsq[wavno] = (hmdiff[wavno] - mhdiff)*(hmdiff[wavno] - mhdiff);	
		}
		
		
		mad = median(nwi, adslopes);
		stdev = sqrt(mean(nwav, hdsq));
		
		fprintf(stdout,"%lf \n", stdev);
		
		endloop:
		loop = 1;
		ns++;
	}
	
	endpoint:
	fclose (fpin);
	fprintf(stderr, "Number of 1 Hz records: %d \n", ns);
	return EXIT_SUCCESS;
}

double mean(int nw, double htd[]) {
    double	sum=0;
    int		i;
    
    for(i=0; i<nw; i++)
        sum+=htd[i];
    return((double)sum/nw);
}
 
double median(int nw, double sld[]) {
    double 	temp;
    int 	i, j;
    // the following two loops sort the array x in ascending order
    for(i=0; i<nw-1; i++) {
        for(j=i+1; j<nw; j++) {
            if(sld[j] < sld[i]) {
                // swap elements
                temp = sld[i];
                sld[i] = sld[j];
                sld[j] = temp;
            }
        }
    }
 
    if(nw%2==0) {
        // if there is an even number of elements, return mean of the two elements in the middle
        return((sld[nw/2] + sld[nw/2 - 1]) / 2.0);
    } else {
        // else return the element in the middle
        return sld[nw/2];
    }
}
